@@ -37,6 +37,9 @@ import java.util.Collections;
 
 import static org.apache.avro.Schema.Type.INT;
 import static org.apache.avro.Schema.Type.LONG;
+import static org.apache.avro.Schema.Type.STRING;
+import static org.apache.avro.SchemaCompatibility.checkReaderWriterCompatibility;
+import static org.apache.avro.SchemaCompatibility.SchemaCompatibilityType.COMPATIBLE;
 import static org.apache.parquet.avro.AvroTestUtil.field;
 import static org.apache.parquet.avro.AvroTestUtil.optionalField;
 import static org.apache.parquet.avro.AvroTestUtil.primitive;
@@ -582,6 +585,32 @@ public class TestAvroSchemaConverter {
             "}\n");
   }
 
+  @Test
+  public void testParquetInt96AsFixed12AvroType() throws Exception {
+    Configuration enableInt96ReadingConfig = new Configuration();
+    enableInt96ReadingConfig.setBoolean(AvroReadSupport.READ_INT96_AS_FIXED, true);
+
+    Schema schema = Schema.createRecord("myrecord", null, null, false);
+    Schema int96schema = Schema.createFixed("INT96", "INT96 represented as byte[12]", null, 12);
+    schema.setFields(Collections.singletonList(
+      new Schema.Field("int96_field", int96schema, null, null)));
+
+    testParquetToAvroConversion(enableInt96ReadingConfig, schema, "message myrecord {\n" +
+      "  required int96 int96_field;\n" +
+      "}\n");
+  }
+
+  @Test
+  public void testParquetInt96DefaultFail() throws Exception {
+    Schema schema = Schema.createRecord("myrecord", null, null, false);
+
+    MessageType parquetSchemaWithInt96 = MessageTypeParser.parseMessageType("message myrecord {\n  required int96 int96_field;\n}\n");
+
+    assertThrows("INT96 is deprecated. As interim enable READ_INT96_AS_FIXED  flag to read as byte array.",
+      IllegalArgumentException.class,
+      () -> new AvroSchemaConverter().convert(parquetSchemaWithInt96));
+  }
+
   @Test
   public void testDateType() throws Exception {
     Schema date = LogicalTypes.date().addToSchema(Schema.create(INT));
@@ -766,6 +795,35 @@ public class TestAvroSchemaConverter {
     testParquetToAvroConversion(NEW_BEHAVIOR, schema, parquetSchema);
   }
 
+  @Test
+  public void testUUIDType() throws Exception {
+    Schema fromAvro = Schema.createRecord("myrecord", null, null, false,
+        Arrays.asList(new Schema.Field("uuid", LogicalTypes.uuid().addToSchema(Schema.create(STRING)), null, null)));
+    String parquet = "message myrecord {\n" +
+        "  required binary uuid (STRING);\n" +
+        "}\n";
+    Schema toAvro = Schema.createRecord("myrecord", null, null, false,
+        Arrays.asList(new Schema.Field("uuid", Schema.create(STRING), null, null)));
+
+    testAvroToParquetConversion(fromAvro, parquet);
+    testParquetToAvroConversion(toAvro, parquet);
+
+    assertEquals(COMPATIBLE, checkReaderWriterCompatibility(fromAvro, toAvro).getType());
+  }
+
+  @Test
+  public void testUUIDTypeWithParquetUUID() throws Exception {
+    Schema uuid = LogicalTypes.uuid().addToSchema(Schema.create(STRING));
+    Schema expected = Schema.createRecord("myrecord", null, null, false,
+        Arrays.asList(new Schema.Field("uuid", uuid, null, null)));
+
+    testRoundTripConversion(AvroTestUtil.conf(AvroWriteSupport.WRITE_PARQUET_UUID, true),
+        expected,
+        "message myrecord {\n" +
+            "  required fixed_len_byte_array(16) uuid (UUID);\n" +
+            "}\n");
+  }
+
   public static Schema optional(Schema original) {
     return Schema.createUnion(Lists.newArrayList(
         Schema.create(Schema.Type.NULL),
