@@ -38,7 +38,6 @@ import org.apache.parquet.example.DummyRecordConverter;
 import org.apache.parquet.io.api.Binary;
 import org.apache.parquet.schema.MessageType;
 import org.apache.parquet.schema.MessageTypeParser;
-import org.apache.parquet.schema.PrimitiveType.PrimitiveTypeName;
 import org.apache.parquet.schema.Types;
 import org.junit.Test;
 import org.slf4j.Logger;
@@ -168,7 +167,7 @@ public class TestMemColumn {
       assertEquals("r row " + i, r, columnReader.getCurrentRepetitionLevel());
       assertEquals("d row " + i, d, columnReader.getCurrentDefinitionLevel());
       if (d == 2) {
-        assertEquals("data row " + i, (long)i, columnReader.getLong());
+        assertEquals("data row " + i, i, columnReader.getLong());
       }
       columnReader.consume();
       ++ i;
