@@ -21,6 +21,9 @@ package org.apache.parquet.bytes;
 
 import org.junit.Assert;
 import org.junit.Test;
+
+import static org.junit.Assert.assertEquals;
+
 import java.io.EOFException;
 import java.io.IOException;
 import java.nio.ByteBuffer;
@@ -30,6 +33,7 @@ import java.util.List;
 import java.util.concurrent.Callable;
 
 public abstract class TestByteBufferInputStreams {
+  static final int DATA_LENGTH = 35;
 
   protected abstract ByteBufferInputStream newStream();
   protected abstract void checkOriginalData();
@@ -172,12 +176,7 @@ public abstract class TestByteBufferInputStreams {
     }
 
     assertThrows("Should throw EOFException at end of stream",
-        EOFException.class, new Callable<Integer>() {
-          @Override
-          public Integer call() throws IOException {
-            return stream.read();
-          }
-        });
+        EOFException.class, (Callable<Integer>) stream::read);
 
     checkOriginalData();
   }
@@ -229,12 +228,7 @@ public abstract class TestByteBufferInputStreams {
     Assert.assertEquals("Should consume all buffers", length, stream.position());
 
     assertThrows("Should throw EOFException when empty",
-        EOFException.class, new Callable<List<ByteBuffer>>() {
-          @Override
-          public List<ByteBuffer> call() throws Exception {
-            return stream.sliceBuffers(length);
-          }
-        });
+        EOFException.class, (Callable<List<ByteBuffer>>) () -> stream.sliceBuffers(length));
 
     ByteBufferInputStream copy = ByteBufferInputStream.wrap(buffers);
     for (int i = 0; i < length; i += 1) {
@@ -361,12 +355,9 @@ public abstract class TestByteBufferInputStreams {
 
     final int length = stream2.available();
     assertThrows("Should throw when out of bytes",
-        EOFException.class, new Callable() {
-          @Override
-          public Object call() throws Exception {
-            stream2.skipFully(length + 10);
-            return null;
-          }
+        EOFException.class, () -> {
+          stream2.skipFully(length + 10);
+          return null;
         });
   }
 
@@ -495,12 +486,9 @@ public abstract class TestByteBufferInputStreams {
     final ByteBufferInputStream stream = newStream();
 
     assertThrows("Should throw an error for reset() without mark()",
-        IOException.class, new Callable() {
-          @Override
-          public Object call() throws Exception {
-            stream.reset();
-            return null;
-          }
+        IOException.class, () -> {
+          stream.reset();
+          return null;
         });
   }
 
@@ -545,12 +533,9 @@ public abstract class TestByteBufferInputStreams {
     Assert.assertEquals("Should read 6 bytes", 6, stream.read(new byte[6]));
 
     assertThrows("Should throw an error for reset() after limit",
-        IOException.class, new Callable() {
-          @Override
-          public Object call() throws Exception {
-            stream.reset();
-            return null;
-          }
+        IOException.class, () -> {
+          stream.reset();
+          return null;
         });
   }
 
@@ -564,15 +549,22 @@ public abstract class TestByteBufferInputStreams {
     stream.reset();
 
     assertThrows("Should throw an error for double reset()",
-        IOException.class, new Callable() {
-          @Override
-          public Object call() throws Exception {
-            stream.reset();
-            return null;
-          }
+        IOException.class, () -> {
+          stream.reset();
+          return null;
         });
   }
 
+  @Test
+  public void testToByteBuffer() {
+    final ByteBufferInputStream stream = newStream();
+
+    ByteBuffer buffer = stream.toByteBuffer();
+    for (int i = 0; i < DATA_LENGTH; ++i) {
+      assertEquals(i, buffer.get());
+    }
+  }
+
   /**
    * A convenience method to avoid a large number of @Test(expected=...) tests
    * @param message A String message to describe this assertion
