@@ -20,11 +20,12 @@ package org.apache.parquet.format.converter;
 
 import static java.util.Optional.empty;
 
-import static java.util.Optional.empty;
 import static java.util.Optional.of;
 import static org.apache.parquet.format.Util.readFileMetaData;
+import static org.apache.parquet.format.Util.writeColumnMetaData;
 import static org.apache.parquet.format.Util.writePageHeader;
 
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -44,8 +45,24 @@ import java.util.concurrent.ConcurrentHashMap;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.parquet.CorruptStatistics;
 import org.apache.parquet.ParquetReadOptions;
+import org.apache.parquet.Preconditions;
+import org.apache.parquet.bytes.ByteBufferInputStream;
 import org.apache.parquet.column.ParquetProperties;
 import org.apache.parquet.column.statistics.BinaryStatistics;
+import org.apache.parquet.column.values.bloomfilter.BloomFilter;
+import org.apache.parquet.crypto.AesCipher;
+import org.apache.parquet.crypto.AesGcmEncryptor;
+import org.apache.parquet.crypto.InternalColumnEncryptionSetup;
+import org.apache.parquet.crypto.InternalFileDecryptor;
+import org.apache.parquet.crypto.InternalFileEncryptor;
+import org.apache.parquet.crypto.ModuleCipherFactory.ModuleType;
+import org.apache.parquet.crypto.ParquetCryptoRuntimeException;
+import org.apache.parquet.crypto.TagVerificationException;
+import org.apache.parquet.format.BlockCipher;
+import org.apache.parquet.format.BloomFilterAlgorithm;
+import org.apache.parquet.format.BloomFilterCompression;
+import org.apache.parquet.format.BloomFilterHash;
+import org.apache.parquet.format.BloomFilterHeader;
 import org.apache.parquet.format.BsonType;
 import org.apache.parquet.format.CompressionCodec;
 import org.apache.parquet.format.DateType;
@@ -61,13 +78,17 @@ import org.apache.parquet.format.MilliSeconds;
 import org.apache.parquet.format.NanoSeconds;
 import org.apache.parquet.format.NullType;
 import org.apache.parquet.format.PageEncodingStats;
+import org.apache.parquet.format.SplitBlockAlgorithm;
 import org.apache.parquet.format.StringType;
 import org.apache.parquet.format.TimeType;
 import org.apache.parquet.format.TimeUnit;
 import org.apache.parquet.format.TimestampType;
+import org.apache.parquet.format.Uncompressed;
+import org.apache.parquet.format.XxHash;
 import org.apache.parquet.hadoop.metadata.ColumnPath;
 import org.apache.parquet.format.BoundaryOrder;
 import org.apache.parquet.format.ColumnChunk;
+import org.apache.parquet.format.ColumnCryptoMetaData;
 import org.apache.parquet.format.ColumnIndex;
 import org.apache.parquet.format.ColumnMetaData;
 import org.apache.parquet.format.ColumnOrder;
@@ -76,6 +97,7 @@ import org.apache.parquet.format.DataPageHeader;
 import org.apache.parquet.format.DataPageHeaderV2;
 import org.apache.parquet.format.DictionaryPageHeader;
 import org.apache.parquet.format.Encoding;
+import org.apache.parquet.format.EncryptionWithColumnKey;
 import org.apache.parquet.format.FieldRepetitionType;
 import org.apache.parquet.format.FileMetaData;
 import org.apache.parquet.format.KeyValue;
@@ -88,6 +110,7 @@ import org.apache.parquet.format.SchemaElement;
 import org.apache.parquet.format.Statistics;
 import org.apache.parquet.format.Type;
 import org.apache.parquet.format.TypeDefinedOrder;
+import org.apache.parquet.format.UUIDType;
 import org.apache.parquet.hadoop.metadata.BlockMetaData;
 import org.apache.parquet.hadoop.metadata.ColumnChunkMetaData;
 import org.apache.parquet.hadoop.metadata.CompressionCodecName;
@@ -97,9 +120,11 @@ import org.apache.parquet.internal.column.columnindex.BinaryTruncator;
 import org.apache.parquet.internal.column.columnindex.ColumnIndexBuilder;
 import org.apache.parquet.internal.column.columnindex.OffsetIndexBuilder;
 import org.apache.parquet.internal.hadoop.metadata.IndexReference;
+import org.apache.parquet.io.InvalidFileOffsetException;
 import org.apache.parquet.io.ParquetDecodingException;
 import org.apache.parquet.io.api.Binary;
 import org.apache.parquet.schema.ColumnOrder.ColumnOrderName;
+import org.apache.parquet.schema.LogicalTypeAnnotation.UUIDLogicalTypeAnnotation;
 import org.apache.parquet.schema.GroupType;
 import org.apache.parquet.schema.MessageType;
 import org.apache.parquet.schema.OriginalType;
@@ -169,12 +194,31 @@ public class ParquetMetadataConverter {
       cachedEncodingSets = new ConcurrentHashMap<Set<org.apache.parquet.column.Encoding>, Set<org.apache.parquet.column.Encoding>>();
 
   public FileMetaData toParquetMetadata(int currentVersion, ParquetMetadata parquetMetadata) {
+    return toParquetMetadata(currentVersion, parquetMetadata, null);
+  }
+
+  public FileMetaData toParquetMetadata(int currentVersion, ParquetMetadata parquetMetadata,
+      InternalFileEncryptor fileEncryptor) {
     List<BlockMetaData> blocks = parquetMetadata.getBlocks();
     List<RowGroup> rowGroups = new ArrayList<RowGroup>();
     long numRows = 0;
+    long preBlockStartPos = 0;
+    long preBlockCompressedSize = 0;
     for (BlockMetaData block : blocks) {
       numRows += block.getRowCount();
-      addRowGroup(parquetMetadata, rowGroups, block);
+      long blockStartPos = block.getStartingPos();
+      // first block
+      if (blockStartPos == 4) {
+        preBlockStartPos = 0;
+        preBlockCompressedSize = 0;
+      }
+      if (preBlockStartPos != 0) {
+        Preconditions.checkState(blockStartPos >= preBlockStartPos + preBlockCompressedSize,
+          "Invalid block starting position:" + blockStartPos);
+      }
+      preBlockStartPos = blockStartPos;
+      preBlockCompressedSize = block.getCompressedSize();
+      addRowGroup(parquetMetadata, rowGroups, block, fileEncryptor);
     }
     FileMetaData fileMetaData = new FileMetaData(
         currentVersion,
@@ -445,20 +489,40 @@ public class ParquetMetadataConverter {
       return of(LogicalType.BSON(new BsonType()));
     }
 
+    @Override
+    public Optional<LogicalType> visit(UUIDLogicalTypeAnnotation uuidLogicalType) {
+      return of(LogicalType.UUID(new UUIDType()));
+    }
+
     @Override
     public Optional<LogicalType> visit(LogicalTypeAnnotation.IntervalLogicalTypeAnnotation intervalLogicalType) {
       return of(LogicalType.UNKNOWN(new NullType()));
     }
   }
 
-  private void addRowGroup(ParquetMetadata parquetMetadata, List<RowGroup> rowGroups, BlockMetaData block) {
+  private void addRowGroup(ParquetMetadata parquetMetadata, List<RowGroup> rowGroups, BlockMetaData block,
+      InternalFileEncryptor fileEncryptor) {
+
     //rowGroup.total_byte_size = ;
     List<ColumnChunkMetaData> columns = block.getColumns();
     List<ColumnChunk> parquetColumns = new ArrayList<ColumnChunk>();
+    int rowGroupOrdinal = rowGroups.size();
+    int columnOrdinal = -1;
+    ByteArrayOutputStream tempOutStream = null;
     for (ColumnChunkMetaData columnMetaData : columns) {
       ColumnChunk columnChunk = new ColumnChunk(columnMetaData.getFirstDataPageOffset()); // verify this is the right offset
       columnChunk.file_path = block.getPath(); // they are in the same file for now
-      columnChunk.meta_data = new ColumnMetaData(
+      InternalColumnEncryptionSetup columnSetup = null;
+      boolean writeCryptoMetadata = false;
+      boolean encryptMetaData = false;
+      ColumnPath path = columnMetaData.getPath();
+      if (null != fileEncryptor) {
+        columnOrdinal++;
+        columnSetup = fileEncryptor.getColumnSetup(path, false, columnOrdinal);
+        writeCryptoMetadata = columnSetup.isEncrypted();
+        encryptMetaData = fileEncryptor.encryptColumnMetaData(columnSetup);
+      }
+      ColumnMetaData metaData = new ColumnMetaData(
           getType(columnMetaData.getType()),
           toFormatEncodings(columnMetaData.getEncodings()),
           Arrays.asList(columnMetaData.getPath().toArray()),
@@ -467,13 +531,50 @@ public class ParquetMetadataConverter {
           columnMetaData.getTotalUncompressedSize(),
           columnMetaData.getTotalSize(),
           columnMetaData.getFirstDataPageOffset());
-      columnChunk.meta_data.dictionary_page_offset = columnMetaData.getDictionaryPageOffset();
-      if (!columnMetaData.getStatistics().isEmpty()) {
-        columnChunk.meta_data.setStatistics(toParquetStatistics(columnMetaData.getStatistics(), this.statisticsTruncateLength));
+      if (columnMetaData.getEncodingStats() != null && columnMetaData.getEncodingStats().hasDictionaryPages()) {
+        metaData.setDictionary_page_offset(columnMetaData.getDictionaryPageOffset());
+      }
+      long bloomFilterOffset = columnMetaData.getBloomFilterOffset();
+      if (bloomFilterOffset >= 0) {
+        metaData.setBloom_filter_offset(bloomFilterOffset);
+      }
+      if (columnMetaData.getStatistics() != null && !columnMetaData.getStatistics().isEmpty()) {
+        metaData.setStatistics(toParquetStatistics(columnMetaData.getStatistics(), this.statisticsTruncateLength));
       }
       if (columnMetaData.getEncodingStats() != null) {
-        columnChunk.meta_data.setEncoding_stats(convertEncodingStats(columnMetaData.getEncodingStats()));
+        metaData.setEncoding_stats(convertEncodingStats(columnMetaData.getEncodingStats()));
       }
+
+      if (!encryptMetaData) {
+        columnChunk.setMeta_data(metaData);
+      }  else {
+        // Serialize and encrypt ColumnMetadata separately
+        byte[] columnMetaDataAAD = AesCipher.createModuleAAD(fileEncryptor.getFileAAD(),
+            ModuleType.ColumnMetaData, rowGroupOrdinal, columnSetup.getOrdinal(), -1);
+        if (null == tempOutStream) {
+          tempOutStream = new ByteArrayOutputStream();
+        }  else {
+          tempOutStream.reset();
+        }
+        try {
+          writeColumnMetaData(metaData, tempOutStream, columnSetup.getMetaDataEncryptor(), columnMetaDataAAD);
+        } catch (IOException e) {
+          throw new ParquetCryptoRuntimeException("Failed to serialize and encrypt ColumnMetadata for " +
+                                                  columnMetaData.getPath(), e);
+        }
+        columnChunk.setEncrypted_column_metadata(tempOutStream.toByteArray());
+        // Keep redacted metadata version for old readers
+        if (!fileEncryptor.isFooterEncrypted()) {
+          ColumnMetaData metaDataRedacted  = metaData.deepCopy();
+          if (metaDataRedacted.isSetStatistics()) metaDataRedacted.unsetStatistics();
+          if (metaDataRedacted.isSetEncoding_stats()) metaDataRedacted.unsetEncoding_stats();
+          columnChunk.setMeta_data(metaDataRedacted);
+        }
+      }
+      if (writeCryptoMetadata) {
+        columnChunk.setCrypto_metadata(columnSetup.getColumnCryptoMetaData());
+      }
+
 //      columnChunk.meta_data.index_page_offset = ;
 //      columnChunk.meta_data.key_value_metadata = ; // nothing yet
 
@@ -491,6 +592,9 @@ public class ParquetMetadataConverter {
       parquetColumns.add(columnChunk);
     }
     RowGroup rowGroup = new RowGroup(parquetColumns, block.getTotalByteSize(), block.getRowCount());
+    rowGroup.setFile_offset(block.getStartingPos());
+    rowGroup.setTotal_compressed_size(block.getCompressedSize());
+    rowGroup.setOrdinal((short) rowGroupOrdinal);
     rowGroups.add(rowGroup);
   }
 
@@ -822,6 +926,11 @@ public class ParquetMetadataConverter {
           return of(SortOrder.UNSIGNED);
         }
 
+        @Override
+        public Optional<SortOrder> visit(UUIDLogicalTypeAnnotation uuidLogicalType) {
+          return of(SortOrder.UNSIGNED);
+        }
+
         @Override
         public Optional<SortOrder> visit(LogicalTypeAnnotation.JsonLogicalTypeAnnotation jsonLogicalType) {
           return of(SortOrder.UNSIGNED);
@@ -998,6 +1107,8 @@ public class ParquetMetadataConverter {
       case TIMESTAMP:
         TimestampType timestamp = type.getTIMESTAMP();
         return LogicalTypeAnnotation.timestampType(timestamp.isAdjustedToUTC, convertTimeUnit(timestamp.unit));
+      case UUID:
+        return LogicalTypeAnnotation.uuidType();
       default:
         throw new RuntimeException("Unknown logical type " + type);
     }
@@ -1130,27 +1241,111 @@ public class ParquetMetadataConverter {
   static FileMetaData filterFileMetaDataByMidpoint(FileMetaData metaData, RangeMetadataFilter filter) {
     List<RowGroup> rowGroups = metaData.getRow_groups();
     List<RowGroup> newRowGroups = new ArrayList<RowGroup>();
+    long preStartIndex = 0;
+    long preCompressedSize = 0;
+    boolean firstColumnWithMetadata = true;
+    if (rowGroups != null && rowGroups.size() > 0) {
+      firstColumnWithMetadata = rowGroups.get(0).getColumns().get(0).isSetMeta_data();
+    }
     for (RowGroup rowGroup : rowGroups) {
       long totalSize = 0;
-      long startIndex = getOffset(rowGroup.getColumns().get(0));
-      for (ColumnChunk col : rowGroup.getColumns()) {
-        totalSize += col.getMeta_data().getTotal_compressed_size();
+      long startIndex;
+      ColumnChunk columnChunk = rowGroup.getColumns().get(0);
+      if (firstColumnWithMetadata) {
+        startIndex = getOffset(columnChunk);
+      } else {
+        assert rowGroup.isSetFile_offset();
+        assert rowGroup.isSetTotal_compressed_size();
+
+        //the file_offset of first block always holds the truth, while other blocks don't :
+        //see PARQUET-2078 for details
+        startIndex = rowGroup.getFile_offset();
+        if (invalidFileOffset(startIndex, preStartIndex, preCompressedSize)) {
+          //first row group's offset is always 4
+          if (preStartIndex == 0) {
+            startIndex = 4;
+          } else {
+            // use minStartIndex(imprecise in case of padding, but good enough for filtering)
+            startIndex = preStartIndex + preCompressedSize;
+          }
+        }
+        preStartIndex = startIndex;
+        preCompressedSize = rowGroup.getTotal_compressed_size();
       }
+
+      if (rowGroup.isSetTotal_compressed_size()) {
+        totalSize = rowGroup.getTotal_compressed_size();
+      } else {
+        for (ColumnChunk col : rowGroup.getColumns()) {
+          totalSize += col.getMeta_data().getTotal_compressed_size();
+        }
+      }
+
       long midPoint = startIndex + totalSize / 2;
       if (filter.contains(midPoint)) {
         newRowGroups.add(rowGroup);
       }
     }
+
     metaData.setRow_groups(newRowGroups);
     return metaData;
   }
 
+  private static boolean invalidFileOffset(long startIndex, long preStartIndex, long preCompressedSize) {
+    boolean invalid = false;
+    assert preStartIndex <= startIndex;
+    // checking the first rowGroup
+    if (preStartIndex == 0 && startIndex != 4) {
+      invalid = true;
+      return invalid;
+    }
+
+    //calculate start index for other blocks
+    long minStartIndex = preStartIndex + preCompressedSize;
+    if (startIndex < minStartIndex) {
+      // a bad offset detected, try first column's offset
+      // can not use minStartIndex in case of padding
+      invalid = true;
+    }
+
+    return invalid;
+  }
+
   // Visible for testing
   static FileMetaData filterFileMetaDataByStart(FileMetaData metaData, OffsetMetadataFilter filter) {
     List<RowGroup> rowGroups = metaData.getRow_groups();
     List<RowGroup> newRowGroups = new ArrayList<RowGroup>();
+    long preStartIndex = 0;
+    long preCompressedSize = 0;
+    boolean firstColumnWithMetadata = true;
+    if (rowGroups != null && rowGroups.size() > 0) {
+      firstColumnWithMetadata = rowGroups.get(0).getColumns().get(0).isSetMeta_data();
+    }
     for (RowGroup rowGroup : rowGroups) {
-      long startIndex = getOffset(rowGroup.getColumns().get(0));
+      long startIndex;
+      ColumnChunk columnChunk = rowGroup.getColumns().get(0);
+      if (firstColumnWithMetadata) {
+        startIndex = getOffset(columnChunk);
+      } else {
+        assert rowGroup.isSetFile_offset();
+        assert rowGroup.isSetTotal_compressed_size();
+
+        //the file_offset of first block always holds the truth, while other blocks don't :
+        //see PARQUET-2078 for details
+        startIndex = rowGroup.getFile_offset();
+        if (invalidFileOffset(startIndex, preStartIndex, preCompressedSize)) {
+          //first row group's offset is always 4
+          if (preStartIndex == 0) {
+            startIndex = 4;
+          } else {
+            throw new InvalidFileOffsetException("corrupted RowGroup.file_offset found, " +
+              "please use file range instead of block offset for split.");
+          }
+        }
+        preStartIndex = startIndex;
+        preCompressedSize = rowGroup.getTotal_compressed_size();
+      }
+
       if (filter.contains(startIndex)) {
         newRowGroups.add(rowGroup);
       }
@@ -1160,8 +1355,12 @@ public class ParquetMetadataConverter {
   }
 
   static long getOffset(RowGroup rowGroup) {
+    if (rowGroup.isSetFile_offset()) {
+      return rowGroup.getFile_offset();
+    }
     return getOffset(rowGroup.getColumns().get(0));
   }
+
   // Visible for testing
   static long getOffset(ColumnChunk columnChunk) {
     ColumnMetaData md = columnChunk.getMeta_data();
@@ -1172,69 +1371,188 @@ public class ParquetMetadataConverter {
     return offset;
   }
 
+  private static void verifyFooterIntegrity(InputStream from, InternalFileDecryptor fileDecryptor,
+      int combinedFooterLength) throws IOException {
+
+    byte[] nonce = new byte[AesCipher.NONCE_LENGTH];
+    from.read(nonce);
+    byte[] gcmTag = new byte[AesCipher.GCM_TAG_LENGTH];
+    from.read(gcmTag);
+
+    AesGcmEncryptor footerSigner =  fileDecryptor.createSignedFooterEncryptor();
+
+    byte[] footerAndSignature = ((ByteBufferInputStream) from).slice(0).array();
+    int footerSignatureLength = AesCipher.NONCE_LENGTH + AesCipher.GCM_TAG_LENGTH;
+    byte[] serializedFooter = new byte[combinedFooterLength - footerSignatureLength];
+    System.arraycopy(footerAndSignature, 0, serializedFooter, 0, serializedFooter.length);
+
+    byte[] signedFooterAAD = AesCipher.createFooterAAD(fileDecryptor.getFileAAD());
+    byte[] encryptedFooterBytes = footerSigner.encrypt(false, serializedFooter, nonce, signedFooterAAD);
+    byte[] calculatedTag = new byte[AesCipher.GCM_TAG_LENGTH];
+    System.arraycopy(encryptedFooterBytes, encryptedFooterBytes.length - AesCipher.GCM_TAG_LENGTH,
+        calculatedTag, 0, AesCipher.GCM_TAG_LENGTH);
+    if (!Arrays.equals(gcmTag, calculatedTag)) {
+      throw new TagVerificationException("Signature mismatch in plaintext footer");
+    }
+  }
+
   public ParquetMetadata readParquetMetadata(final InputStream from, MetadataFilter filter) throws IOException {
+    return readParquetMetadata(from, filter, null, false, 0);
+  }
+
+  public ParquetMetadata readParquetMetadata(final InputStream from, MetadataFilter filter,
+      final InternalFileDecryptor fileDecryptor, final boolean encryptedFooter,
+      final int combinedFooterLength) throws IOException {
+
+    final BlockCipher.Decryptor footerDecryptor = (encryptedFooter? fileDecryptor.fetchFooterDecryptor() : null);
+    final byte[] encryptedFooterAAD = (encryptedFooter? AesCipher.createFooterAAD(fileDecryptor.getFileAAD()) : null);
+
     FileMetaData fileMetaData = filter.accept(new MetadataFilterVisitor<FileMetaData, IOException>() {
       @Override
       public FileMetaData visit(NoFilter filter) throws IOException {
-        return readFileMetaData(from);
+        return readFileMetaData(from, footerDecryptor, encryptedFooterAAD);
       }
 
       @Override
       public FileMetaData visit(SkipMetadataFilter filter) throws IOException {
-        return readFileMetaData(from, true);
+        return readFileMetaData(from, true, footerDecryptor, encryptedFooterAAD);
       }
 
       @Override
       public FileMetaData visit(OffsetMetadataFilter filter) throws IOException {
-        return filterFileMetaDataByStart(readFileMetaData(from), filter);
+        return filterFileMetaDataByStart(readFileMetaData(from, footerDecryptor, encryptedFooterAAD), filter);
       }
 
       @Override
       public FileMetaData visit(RangeMetadataFilter filter) throws IOException {
-        return filterFileMetaDataByMidpoint(readFileMetaData(from), filter);
+        return filterFileMetaDataByMidpoint(readFileMetaData(from, footerDecryptor, encryptedFooterAAD), filter);
       }
     });
     LOG.debug("{}", fileMetaData);
-    ParquetMetadata parquetMetadata = fromParquetMetadata(fileMetaData);
+
+    if (!encryptedFooter && null != fileDecryptor) {
+      if (!fileMetaData.isSetEncryption_algorithm()) { // Plaintext file
+        fileDecryptor.setPlaintextFile();
+        // Done to detect files that were not encrypted by mistake
+        if (!fileDecryptor.plaintextFilesAllowed()) {
+          throw new ParquetCryptoRuntimeException("Applying decryptor on plaintext file");
+        }
+      } else {  // Encrypted file with plaintext footer
+        // if no fileDecryptor, can still read plaintext columns
+        fileDecryptor.setFileCryptoMetaData(fileMetaData.getEncryption_algorithm(), false,
+            fileMetaData.getFooter_signing_key_metadata());
+        if (fileDecryptor.checkFooterIntegrity()) {
+          verifyFooterIntegrity(from, fileDecryptor, combinedFooterLength);
+        }
+      }
+    }
+
+    ParquetMetadata parquetMetadata = fromParquetMetadata(fileMetaData, fileDecryptor, encryptedFooter);
     if (LOG.isDebugEnabled()) LOG.debug(ParquetMetadata.toPrettyJSON(parquetMetadata));
     return parquetMetadata;
   }
 
+  public ColumnChunkMetaData buildColumnChunkMetaData(ColumnMetaData metaData, ColumnPath columnPath, PrimitiveType type, String createdBy) {
+    return ColumnChunkMetaData.get(
+        columnPath,
+        type,
+        fromFormatCodec(metaData.codec),
+        convertEncodingStats(metaData.getEncoding_stats()),
+        fromFormatEncodings(metaData.encodings),
+        fromParquetStatistics(
+            createdBy,
+            metaData.statistics,
+            type),
+        metaData.data_page_offset,
+        metaData.dictionary_page_offset,
+        metaData.num_values,
+        metaData.total_compressed_size,
+        metaData.total_uncompressed_size);
+  }
+
   public ParquetMetadata fromParquetMetadata(FileMetaData parquetMetadata) throws IOException {
+    return fromParquetMetadata(parquetMetadata, null, false);
+  }
+
+  public ParquetMetadata fromParquetMetadata(FileMetaData parquetMetadata,
+      InternalFileDecryptor fileDecryptor, boolean encryptedFooter) throws IOException {
     MessageType messageType = fromParquetSchema(parquetMetadata.getSchema(), parquetMetadata.getColumn_orders());
     List<BlockMetaData> blocks = new ArrayList<BlockMetaData>();
     List<RowGroup> row_groups = parquetMetadata.getRow_groups();
+
     if (row_groups != null) {
       for (RowGroup rowGroup : row_groups) {
         BlockMetaData blockMetaData = new BlockMetaData();
         blockMetaData.setRowCount(rowGroup.getNum_rows());
         blockMetaData.setTotalByteSize(rowGroup.getTotal_byte_size());
+        // not set in legacy files
+        if (rowGroup.isSetOrdinal()) {
+          blockMetaData.setOrdinal(rowGroup.getOrdinal());
+        }
         List<ColumnChunk> columns = rowGroup.getColumns();
         String filePath = columns.get(0).getFile_path();
+        int columnOrdinal = -1;
         for (ColumnChunk columnChunk : columns) {
+          columnOrdinal++;
           if ((filePath == null && columnChunk.getFile_path() != null)
               || (filePath != null && !filePath.equals(columnChunk.getFile_path()))) {
             throw new ParquetDecodingException("all column chunks of the same row group must be in the same file for now");
           }
           ColumnMetaData metaData = columnChunk.meta_data;
-          ColumnPath path = getPath(metaData);
-          ColumnChunkMetaData column = ColumnChunkMetaData.get(
-              path,
-              messageType.getType(path.toArray()).asPrimitiveType(),
-              fromFormatCodec(metaData.codec),
-              convertEncodingStats(metaData.getEncoding_stats()),
-              fromFormatEncodings(metaData.encodings),
-              fromParquetStatistics(
-                  parquetMetadata.getCreated_by(),
-                  metaData.statistics,
-                  messageType.getType(path.toArray()).asPrimitiveType()),
-              metaData.data_page_offset,
-              metaData.dictionary_page_offset,
-              metaData.num_values,
-              metaData.total_compressed_size,
-              metaData.total_uncompressed_size);
+          ColumnCryptoMetaData cryptoMetaData = columnChunk.getCrypto_metadata();
+          ColumnChunkMetaData column = null;
+          ColumnPath columnPath = null;
+          boolean encryptedMetadata = false;
+
+          if (null == cryptoMetaData) { // Plaintext column
+            columnPath = getPath(metaData);
+            if (null != fileDecryptor && !fileDecryptor.plaintextFile()) {
+              // mark this column as plaintext in encrypted file decryptor
+              fileDecryptor.setColumnCryptoMetadata(columnPath, false, false, (byte[]) null, columnOrdinal);
+            }
+          } else {  // Encrypted column
+            boolean encryptedWithFooterKey = cryptoMetaData.isSetENCRYPTION_WITH_FOOTER_KEY();
+            if (encryptedWithFooterKey) { // Column encrypted with footer key
+              if (!encryptedFooter) {
+                throw new ParquetCryptoRuntimeException("Column encrypted with footer key in file with plaintext footer");
+              }
+              if (null == metaData) {
+                throw new ParquetCryptoRuntimeException("ColumnMetaData not set in Encryption with Footer key");
+              }
+              if (null == fileDecryptor) {
+                throw new ParquetCryptoRuntimeException("Column encrypted with footer key: No keys available");
+              }
+              columnPath = getPath(metaData);
+              fileDecryptor.setColumnCryptoMetadata(columnPath, true, true, (byte[]) null, columnOrdinal);
+            }  else { // Column encrypted with column key
+              // setColumnCryptoMetadata triggers KMS interaction, hence delayed until this column is projected
+              encryptedMetadata = true;
+            }
+          }
+
+          String createdBy = parquetMetadata.getCreated_by();
+          if (!encryptedMetadata) { // unencrypted column, or encrypted with footer key
+            column = buildColumnChunkMetaData(metaData, columnPath,
+                messageType.getType(columnPath.toArray()).asPrimitiveType(), createdBy);
+            column.setRowGroupOrdinal(rowGroup.getOrdinal());
+            if (metaData.isSetBloom_filter_offset()) {
+              column.setBloomFilterOffset(metaData.getBloom_filter_offset());
+            }
+          }  else { // column encrypted with column key
+            // Metadata will be decrypted later, if this column is accessed
+            EncryptionWithColumnKey columnKeyStruct = cryptoMetaData.getENCRYPTION_WITH_COLUMN_KEY();
+            List<String> pathList = columnKeyStruct.getPath_in_schema();
+            byte[] columnKeyMetadata = columnKeyStruct.getKey_metadata();
+            columnPath = ColumnPath.get(pathList.toArray(new String[pathList.size()]));
+            byte[] encryptedMetadataBuffer = columnChunk.getEncrypted_column_metadata();
+            column = ColumnChunkMetaData.getWithEncryptedMetadata(this, columnPath,
+                messageType.getType(columnPath.toArray()).asPrimitiveType(), encryptedMetadataBuffer,
+                columnKeyMetadata, fileDecryptor, rowGroup.getOrdinal(), columnOrdinal, createdBy);
+          }
+
           column.setColumnIndexReference(toColumnIndexReference(columnChunk));
           column.setOffsetIndexReference(toOffsetIndexReference(columnChunk));
+
           // TODO
           // index_page_offset
           // key_value_metadata
@@ -1252,7 +1570,7 @@ public class ParquetMetadataConverter {
       }
     }
     return new ParquetMetadata(
-        new org.apache.parquet.hadoop.metadata.FileMetaData(messageType, keyValueMetaData, parquetMetadata.getCreated_by()),
+        new org.apache.parquet.hadoop.metadata.FileMetaData(messageType, keyValueMetaData, parquetMetadata.getCreated_by(), fileDecryptor),
         blocks);
   }
 
@@ -1271,7 +1589,7 @@ public class ParquetMetadataConverter {
   }
 
   private static ColumnPath getPath(ColumnMetaData metaData) {
-    String[] path = metaData.path_in_schema.toArray(new String[metaData.path_in_schema.size()]);
+    String[] path = metaData.path_in_schema.toArray(new String[0]);
     return ColumnPath.get(path);
   }
 
@@ -1453,19 +1771,34 @@ public class ParquetMetadataConverter {
   }
 
   public void writeDataPageV1Header(
-    int uncompressedSize,
-    int compressedSize,
-    int valueCount,
-    org.apache.parquet.column.Encoding rlEncoding,
-    org.apache.parquet.column.Encoding dlEncoding,
-    org.apache.parquet.column.Encoding valuesEncoding,
-    OutputStream to) throws IOException {
+      int uncompressedSize,
+      int compressedSize,
+      int valueCount,
+      org.apache.parquet.column.Encoding rlEncoding,
+      org.apache.parquet.column.Encoding dlEncoding,
+      org.apache.parquet.column.Encoding valuesEncoding,
+      OutputStream to) throws IOException {
+    writeDataPageV1Header(uncompressedSize, compressedSize, valueCount,
+        rlEncoding, dlEncoding, valuesEncoding, to, null, null);
+  }
+
+  public void writeDataPageV1Header(
+      int uncompressedSize,
+      int compressedSize,
+      int valueCount,
+      org.apache.parquet.column.Encoding rlEncoding,
+      org.apache.parquet.column.Encoding dlEncoding,
+      org.apache.parquet.column.Encoding valuesEncoding,
+      OutputStream to,
+      BlockCipher.Encryptor blockEncryptor,
+      byte[] AAD) throws IOException {
     writePageHeader(newDataPageHeader(uncompressedSize,
-      compressedSize,
-      valueCount,
-      rlEncoding,
-      dlEncoding,
-      valuesEncoding), to);
+                                      compressedSize,
+                                      valueCount,
+                                      rlEncoding,
+                                      dlEncoding,
+                                      valuesEncoding),
+                    to, blockEncryptor, AAD);
   }
 
   public void writeDataPageV1Header(
@@ -1477,13 +1810,29 @@ public class ParquetMetadataConverter {
       org.apache.parquet.column.Encoding valuesEncoding,
       int crc,
       OutputStream to) throws IOException {
+    writeDataPageV1Header(uncompressedSize, compressedSize, valueCount,
+        rlEncoding, dlEncoding, valuesEncoding, crc, to, null, null);
+  }
+
+  public void writeDataPageV1Header(
+      int uncompressedSize,
+      int compressedSize,
+      int valueCount,
+      org.apache.parquet.column.Encoding rlEncoding,
+      org.apache.parquet.column.Encoding dlEncoding,
+      org.apache.parquet.column.Encoding valuesEncoding,
+      int crc,
+      OutputStream to,
+      BlockCipher.Encryptor blockEncryptor,
+      byte[] AAD) throws IOException {
     writePageHeader(newDataPageHeader(uncompressedSize,
                                       compressedSize,
                                       valueCount,
                                       rlEncoding,
                                       dlEncoding,
                                       valuesEncoding,
-                                      crc), to);
+                                      crc),
+                    to, blockEncryptor, AAD);
   }
 
   public void writeDataPageV2Header(
@@ -1492,12 +1841,24 @@ public class ParquetMetadataConverter {
       org.apache.parquet.column.Encoding dataEncoding,
       int rlByteLength, int dlByteLength,
       OutputStream to) throws IOException {
+    writeDataPageV2Header(uncompressedSize, compressedSize,
+        valueCount, nullCount, rowCount, dataEncoding,
+        rlByteLength, dlByteLength, to, null, null);
+  }
+
+  public void writeDataPageV2Header(
+      int uncompressedSize, int compressedSize,
+      int valueCount, int nullCount, int rowCount,
+      org.apache.parquet.column.Encoding dataEncoding,
+      int rlByteLength, int dlByteLength,
+      OutputStream to, BlockCipher.Encryptor blockEncryptor,
+      byte[] AAD) throws IOException {
     writePageHeader(
         newDataPageV2Header(
             uncompressedSize, compressedSize,
             valueCount, nullCount, rowCount,
             dataEncoding,
-            rlByteLength, dlByteLength), to);
+            rlByteLength, dlByteLength), to, blockEncryptor, AAD);
   }
 
   private PageHeader newDataPageV2Header(
@@ -1516,20 +1877,36 @@ public class ParquetMetadataConverter {
   }
 
   public void writeDictionaryPageHeader(
-    int uncompressedSize, int compressedSize, int valueCount,
-    org.apache.parquet.column.Encoding valuesEncoding, OutputStream to) throws IOException {
+      int uncompressedSize, int compressedSize, int valueCount,
+      org.apache.parquet.column.Encoding valuesEncoding, OutputStream to) throws IOException {
+    writeDictionaryPageHeader(uncompressedSize, compressedSize, valueCount,
+        valuesEncoding, to, null, null);
+  }
+
+  public void writeDictionaryPageHeader(
+      int uncompressedSize, int compressedSize, int valueCount,
+      org.apache.parquet.column.Encoding valuesEncoding, OutputStream to,
+      BlockCipher.Encryptor blockEncryptor, byte[] AAD) throws IOException {
     PageHeader pageHeader = new PageHeader(PageType.DICTIONARY_PAGE, uncompressedSize, compressedSize);
     pageHeader.setDictionary_page_header(new DictionaryPageHeader(valueCount, getEncoding(valuesEncoding)));
-    writePageHeader(pageHeader, to);
+    writePageHeader(pageHeader, to, blockEncryptor, AAD);
   }
 
   public void writeDictionaryPageHeader(
       int uncompressedSize, int compressedSize, int valueCount,
       org.apache.parquet.column.Encoding valuesEncoding, int crc, OutputStream to) throws IOException {
+    writeDictionaryPageHeader(uncompressedSize, compressedSize, valueCount,
+        valuesEncoding, crc, to, null, null);
+  }
+
+  public void writeDictionaryPageHeader(
+      int uncompressedSize, int compressedSize, int valueCount,
+      org.apache.parquet.column.Encoding valuesEncoding, int crc, OutputStream to,
+      BlockCipher.Encryptor blockEncryptor, byte[] AAD) throws IOException {
     PageHeader pageHeader = new PageHeader(PageType.DICTIONARY_PAGE, uncompressedSize, compressedSize);
     pageHeader.setCrc(crc);
     pageHeader.setDictionary_page_header(new DictionaryPageHeader(valueCount, getEncoding(valuesEncoding)));
-    writePageHeader(pageHeader, to);
+    writePageHeader(pageHeader, to, blockEncryptor, AAD);
   }
 
   private static BoundaryOrder toParquetBoundaryOrder(
@@ -1606,4 +1983,32 @@ public class ParquetMetadataConverter {
     }
     return builder.build();
   }
+
+  public static BloomFilterHeader toBloomFilterHeader(
+    org.apache.parquet.column.values.bloomfilter.BloomFilter bloomFilter) {
+
+    BloomFilterAlgorithm algorithm = null;
+    BloomFilterHash hashStrategy = null;
+    BloomFilterCompression compression = null;
+
+    if (bloomFilter.getAlgorithm() == BloomFilter.Algorithm.BLOCK) {
+      algorithm = BloomFilterAlgorithm.BLOCK(new SplitBlockAlgorithm());
+    }
+
+    if (bloomFilter.getHashStrategy() == BloomFilter.HashStrategy.XXH64) {
+      hashStrategy = BloomFilterHash.XXHASH(new XxHash());
+    }
+
+    if (bloomFilter.getCompression() == BloomFilter.Compression.UNCOMPRESSED) {
+      compression = BloomFilterCompression.UNCOMPRESSED(new Uncompressed());
+    }
+
+    if (algorithm != null && hashStrategy != null && compression != null) {
+      return new BloomFilterHeader(bloomFilter.getBitsetSize(), algorithm, hashStrategy, compression);
+    } else {
+      throw new IllegalArgumentException(String.format("Failed to build thrift structure for BloomFilterHeader," +
+        "algorithm=%s, hash=%s, compression=%s",
+        bloomFilter.getAlgorithm(), bloomFilter.getHashStrategy(), bloomFilter.getCompression()));
+    }
+  }
 }
