@@ -1,4 +1,4 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- * 
+ *
  *   http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -18,14 +18,13 @@
  */
 package org.apache.parquet.hadoop;
 
-import static org.apache.parquet.Preconditions.checkNotNull;
-
 import java.io.Closeable;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Objects;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
@@ -35,6 +34,7 @@ import org.apache.hadoop.fs.Path;
 import org.apache.parquet.ParquetReadOptions;
 import org.apache.parquet.Preconditions;
 import org.apache.parquet.compression.CompressionCodecFactory;
+import org.apache.parquet.crypto.FileDecryptionProperties;
 import org.apache.parquet.filter.UnboundRecordFilter;
 import org.apache.parquet.filter2.compat.FilterCompat;
 import org.apache.parquet.filter2.compat.FilterCompat.Filter;
@@ -109,8 +109,8 @@ public class ParquetReader<T> implements Closeable {
                         ReadSupport<T> readSupport,
                         FilterCompat.Filter filter) throws IOException {
     this(Collections.singletonList((InputFile) HadoopInputFile.fromPath(file, conf)),
-        HadoopReadOptions.builder(conf)
-            .withRecordFilter(checkNotNull(filter, "filter"))
+        HadoopReadOptions.builder(conf, file)
+            .withRecordFilter(Objects.requireNonNull(filter, "filter cannot be null"))
             .build(),
         readSupport);
   }
@@ -182,37 +182,39 @@ public class ParquetReader<T> implements Closeable {
 
     @Deprecated
     private Builder(ReadSupport<T> readSupport, Path path) {
-      this.readSupport = checkNotNull(readSupport, "readSupport");
+      this.readSupport = Objects.requireNonNull(readSupport, "readSupport cannot be null");
       this.file = null;
-      this.path = checkNotNull(path, "path");
+      this.path = Objects.requireNonNull(path, "path cannot be null");
       this.conf = new Configuration();
-      this.optionsBuilder = HadoopReadOptions.builder(conf);
+      this.optionsBuilder = HadoopReadOptions.builder(conf, path);
     }
 
     @Deprecated
     protected Builder(Path path) {
       this.readSupport = null;
       this.file = null;
-      this.path = checkNotNull(path, "path");
+      this.path = Objects.requireNonNull(path, "path cannot be null");
       this.conf = new Configuration();
-      this.optionsBuilder = HadoopReadOptions.builder(conf);
+      this.optionsBuilder = HadoopReadOptions.builder(conf, path);
     }
 
     protected Builder(InputFile file) {
       this.readSupport = null;
-      this.file = checkNotNull(file, "file");
+      this.file = Objects.requireNonNull(file, "file cannot be null");
       this.path = null;
       if (file instanceof HadoopInputFile) {
-        this.conf = ((HadoopInputFile) file).getConfiguration();
+        HadoopInputFile hadoopFile = (HadoopInputFile) file;
+        this.conf = hadoopFile.getConfiguration();
+        optionsBuilder = HadoopReadOptions.builder(conf, hadoopFile.getPath());
       } else {
         this.conf = new Configuration();
+        optionsBuilder = HadoopReadOptions.builder(conf);
       }
-      optionsBuilder = HadoopReadOptions.builder(conf);
     }
 
     // when called, resets options to the defaults from conf
     public Builder<T> withConf(Configuration conf) {
-      this.conf = checkNotNull(conf, "conf");
+      this.conf = Objects.requireNonNull(conf, "conf cannot be null");
 
       // previous versions didn't use the builder, so may set filter before conf. this maintains
       // compatibility for filter. other options are reset by a new conf.
@@ -285,6 +287,16 @@ public class ParquetReader<T> implements Closeable {
       return this;
     }
 
+    public Builder<T> useBloomFilter(boolean useBloomFilter) {
+      optionsBuilder.useBloomFilter(useBloomFilter);
+      return this;
+    }
+
+    public Builder<T> useBloomFilter() {
+      optionsBuilder.useBloomFilter();
+      return this;
+    }
+
     public Builder<T> usePageChecksumVerification() {
       optionsBuilder.usePageChecksumVerification();
       return this;
@@ -299,6 +311,11 @@ public class ParquetReader<T> implements Closeable {
       optionsBuilder.withCodecFactory(codecFactory);
       return this;
     }
+    
+    public Builder<T> withDecryption(FileDecryptionProperties fileDecryptionProperties) {
+      optionsBuilder.withDecryption(fileDecryptionProperties);
+      return this;
+    }
 
     public Builder<T> set(String key, String value) {
       optionsBuilder.set(key, value);
