@@ -113,12 +113,11 @@ public class TestStatistics {
       ParquetProperties.WriterVersion writerVersion = context.version;
       CompressionCodecName codec = CompressionCodecName.UNCOMPRESSED;
 
-      ParquetWriter<Group> writer = new ParquetWriter<Group>(context.fsPath,
+      try (ParquetWriter<Group> writer = new ParquetWriter<Group>(context.fsPath,
           groupWriteSupport, codec, blockSize, pageSize, dictionaryPageSize,
-          enableDictionary, enableValidation, writerVersion, configuration);
-
-      context.write(writer);
-      writer.close();
+          enableDictionary, enableValidation, writerVersion, configuration)) {
+        context.write(writer);
+      }
 
       context.test();
 
@@ -299,9 +298,6 @@ public class TestStatistics {
       if (stats.isEmpty()) {
         // stats are empty if num nulls = 0 and there are no non-null values
         // this happens if stats are not written (e.g., when stats are too big)
-        System.err.println(String.format(
-            "No stats written for page=%s col=%s",
-            page, Arrays.toString(desc.getPath())));
         return;
       }
 
@@ -317,12 +313,6 @@ public class TestStatistics {
       }
 
       Assert.assertEquals(numNulls, stats.getNumNulls());
-
-      System.err.println(String.format(
-          "Validated stats min=%s max=%s nulls=%d for page=%s col=%s",
-          stats.minAsString(),
-          stats.maxAsString(), stats.getNumNulls(), page,
-          Arrays.toString(desc.getPath())));
     }
   }
 
@@ -476,17 +466,18 @@ public class TestStatistics {
       Configuration configuration = new Configuration();
       ParquetMetadata metadata = ParquetFileReader.readFooter(configuration,
           super.fsPath, ParquetMetadataConverter.NO_FILTER);
-      ParquetFileReader reader = new ParquetFileReader(configuration,
+      try (ParquetFileReader reader = new ParquetFileReader(configuration,
         metadata.getFileMetaData(),
         super.fsPath,
         metadata.getBlocks(),
-        metadata.getFileMetaData().getSchema().getColumns());
+        metadata.getFileMetaData().getSchema().getColumns())) {
 
-      PageStatsValidator validator = new PageStatsValidator();
+        PageStatsValidator validator = new PageStatsValidator();
 
-      PageReadStore pageReadStore;
-      while ((pageReadStore = reader.readNextRowGroup()) != null) {
-        validator.validate(metadata.getFileMetaData().getSchema(), pageReadStore);
+        PageReadStore pageReadStore;
+        while ((pageReadStore = reader.readNextRowGroup()) != null) {
+          validator.validate(metadata.getFileMetaData().getSchema(), pageReadStore);
+        }
       }
     }
   }
